@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -405,16 +407,18 @@ SpSubview<eT>::operator_equ_common(const SpBase<eT, T1>& in)
   
   const unwrap_spmat<T1> U(in.get_ref());
   
+  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
+  
   if(U.is_alias(m))
     {
     const SpMat<eT> tmp(U.M);
     
-    return (*this).operator_equ_common(tmp);
+    spglue_merge::subview_merge(*this, tmp);
+    }
+  else
+    {
+    spglue_merge::subview_merge(*this, U.M);
     }
-  
-  arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
-  
-  spglue_merge::subview_merge(*this, U.M);
   
   return *this;
   }
@@ -1077,7 +1081,7 @@ inline
 bool
 SpSubview<eT>::check_overlap(const SpSubview<eT>& x) const
   {
-  const subview<eT>& t = *this;
+  const SpSubview<eT>& t = *this;
   
   if(&t.m != &x.m)
     {
@@ -1760,6 +1764,7 @@ SpSubview_col<eT>::operator=(const Base<eT,T1>& x)
 
 template<typename eT>
 inline
+arma_warn_unused
 const SpOp<SpSubview_col<eT>,spop_htrans>
 SpSubview_col<eT>::t() const
   {
@@ -1770,6 +1775,7 @@ SpSubview_col<eT>::t() const
 
 template<typename eT>
 inline
+arma_warn_unused
 const SpOp<SpSubview_col<eT>,spop_htrans>
 SpSubview_col<eT>::ht() const
   {
@@ -1780,6 +1786,7 @@ SpSubview_col<eT>::ht() const
 
 template<typename eT>
 inline
+arma_warn_unused
 const SpOp<SpSubview_col<eT>,spop_strans>
 SpSubview_col<eT>::st() const
   {
@@ -1866,6 +1873,7 @@ SpSubview_row<eT>::operator=(const Base<eT,T1>& x)
 
 template<typename eT>
 inline
+arma_warn_unused
 const SpOp<SpSubview_row<eT>,spop_htrans>
 SpSubview_row<eT>::t() const
   {
@@ -1876,6 +1884,7 @@ SpSubview_row<eT>::t() const
 
 template<typename eT>
 inline
+arma_warn_unused
 const SpOp<SpSubview_row<eT>,spop_htrans>
 SpSubview_row<eT>::ht() const
   {
@@ -1886,6 +1895,7 @@ SpSubview_row<eT>::ht() const
 
 template<typename eT>
 inline
+arma_warn_unused
 const SpOp<SpSubview_row<eT>,spop_strans>
 SpSubview_row<eT>::st() const
   {
